import logging
import socket
import time
import os

from django.core.management.base import BaseCommand
from django.core.mail import send_mail
from django.conf import settings

import supervisor.xmlrpc
import xmlrpclib
import celery

from celerymon import defaults

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    def __init__(self):
        super(Command, self).__init__()
        self.app = celery.Celery(broker=settings.BROKER_URL)
        self.server = xmlrpclib.ServerProxy(
            defaults.MON_URI,
            transport=supervisor.xmlrpc.SupervisorTransport(
                defaults.MON_SERVER_USER, defaults.MON_SERVER_PASS,
                defaults.MON_SERVER_URL)
        )
        self.queue = None

    @staticmethod
    def send_mail(message):
        if defaults.MON_NOTIFICATION_ENABLED is True:
            send_mail(
                defaults.MON_NOTIFICATION_SUBJECT,
                message, settings.DEFAULT_FROM_EMAIL,
                defaults.MON_NOTIFICATION_EMAILS, fail_silently=True
            )
        logger.warning(message)

    def supervisor_exec(self, method, *args):
        try:
            return getattr(self.server.supervisor, method)(*args)
        except xmlrpclib.Fault:
            return None

    def queue_is_available(self):
        broker = 'celery@' + self.queue
        result = self.app.control.ping([broker])
        if result:
            return result[-1].get(broker) == {'ok': 'pong'}

    def worker_is_run(self):
        info = self.supervisor_exec('getProcessInfo', self.queue)
        if info and info.get('state') == 20:
            return True

    def worker_restart(self):
        self.supervisor_exec('stopProcess', self.queue)
        self.supervisor_exec('startProcess', self.queue)

    def notify_admins(self):
        self.send_mail('Queue "%s" was restarted.' % self.queue)

    def check_all_queues(self):
        for self.queue in defaults.MON_CELERY_WORKERS:
            if not self.queue_is_available() or not self.worker_is_run():
                self.worker_restart()
                self.notify_admins()

    def check_supervisor_state(self):
        try:
            state_data = self.supervisor_exec('getState')
            if state_data and state_data.get('statecode') == 1:
                return True
        except socket.error:
            pass

    def supervisor_restart(self):
        if os.system('service supervisor restart') != 0:
            self.send_mail('Supervisor cannot be restarted')
            raise Exception('Supervisor cannot be restarted')
        self.send_mail('Supervisor was restarted')

    def check_supervisor(self):
        if not self.check_supervisor_state():
            self.supervisor_restart()
            time.sleep(180)

    def handle(self, *args, **options):
        self.check_supervisor()
        self.check_all_queues()



# -*- coding: utf-8 -*-

from django.core.management import call_command
from django.test import TestCase

import supervisor.xmlrpc
import xmlrpclib


class CeleryMonTestCase(TestCase):
    def test_a_is_down(self):
        server = xmlrpclib.ServerProxy(
            'http://127.0.0.1',
            transport=supervisor.xmlrpc.SupervisorTransport(
                None, None, 'unix:///var/run/supervisor.sock')
        )

        self.assertEquals(
            server.supervisor.getProcessInfo('default').get('state'), 20)
        self.assertEquals(
            server.supervisor.getProcessInfo('mail').get('state'), 20)

        server.supervisor.stopProcess('default')

        self.assertEquals(
            server.supervisor.getProcessInfo('default').get('state'), 0)
        self.assertEquals(
            server.supervisor.getProcessInfo('mail').get('state'), 20)

        call_command('check_celery_state')

        self.assertEquals(
            server.supervisor.getProcessInfo('default').get('state'), 20)
        self.assertEquals(
            server.supervisor.getProcessInfo('mail').get('state'), 20)



# -*- encoding: utf-8 -*-

from django.conf import settings


def get_settings(key, default):
    return getattr(settings, key, default)


MON_URI = get_settings('CELERY_MON_URI', 'http://127.0.0.1')
MON_SERVER_URL = get_settings(
    'CELERY_MON_SERVER_URL', 'unix:///var/run/supervisor.sock')
MON_SERVER_USER = get_settings('CELERY_MON_SERVER_USER', None)
MON_SERVER_PASS = get_settings('CELERY_MON_SERVER_PASS', None)

MON_NOTIFICATION_SUBJECT = get_settings(
    'CELERY_MON_NOTIFICATION_SUBJECT', '[celery] worker restart')
MON_NOTIFICATION_ENABLED = get_settings(
    'CELERY_MON_NOTIFICATION_ENABLED', True)
MON_NOTIFICATION_EMAILS = get_settings(
    'CELERY_MON_NOTIFICATION_EMAILS', [n[1] for n in settings.ADMINS])

MON_CELERY_WORKERS = get_settings(
    'CELERY_MON_CELERY_WORKERS', ['default'])



from django import VERSION

DATABASE_ENGINE = 'sqlite3'

SITE_ID = 1

SECRET_KEY = '52b700e3a9dc3f83a410df37777fefbc'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    "kombu.transport.django",
    'djcelery',
    'celerymon',
]

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
)

if VERSION[:2] < (1, 6):
    TEST_RUNNER = 'discover_runner.DiscoverRunner'


BROKER_URL = 'django://'
CELERY_MON_CELERY_WORKERS = ['default', 'mail']

import djcelery
djcelery.setup_loader()
